/*   This file is part of QtFan.

    QtFan is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2

    QtFan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with QtFan.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>

#include "mainwindow.h"
#include "preferencesdialog.h"

QString path;
QString fan_name[8];
QString temp_name[8];
bool firstStart = true;
int trayTemp;

MainWindow::MainWindow()
{
	QWidget *widget = new QWidget;
	setCentralWidget(widget);
	
	createActions();
	createMenus();
	createStatusBar();
	createTrayIcon();

	readSettings();
	findPath();
	
	setWindowTitle(QString::fromUtf8("QtFan"));
	resize(400,400);
	
	// SpinBoxes start
	QSizePolicy spinPolicy(static_cast<QSizePolicy::Policy>(0), static_cast<QSizePolicy::Policy>(0));
	spinPolicy.setHorizontalStretch(0);
	spinPolicy.setVerticalStretch(0);
	for (int i = 0; i < fanList.size(); i++) {
		fanSpin[i] = new QSpinBox;
		fanSpin[i]->setRange(0,240);
		fanSpin[i]->setSingleStep(16);
		fanSpin[i]->setValue(0);
		fanSpin[i]->setSizePolicy(spinPolicy);
	}
	readSpinners();
	for ( int i = 0; i < fanList.size(); i++ )
		connect(fanSpin[i], SIGNAL(valueChanged(int)), this, SLOT(fanCtrl(int)));
	// SpinBoxes end
	

	// Labels start
	for (int i = 0; i < fanList.size(); i++) 
		fanLbl[i] = new QLabel(tr("%1: ").arg(fan_name[i]));
	for (int i = 0; i < tempList.size(); i++)
		tempLbl[i] = new QLabel(tr("%1: ").arg(temp_name[i]));
	// Labels End
	
	// Spacers start
	//QSpacerItem *hspacer = new QSpacerItem(20, 40, QSizePolicy::Expanding, QSizePolicy::Minimum);
	QSpacerItem *vspacer = new QSpacerItem(20,40,QSizePolicy::Minimum, QSizePolicy::Expanding);
	QSpacerItem *vspacer2 = new QSpacerItem(20,40,QSizePolicy::Minimum, QSizePolicy::Expanding);
	// Spacers end
	
	// Lines start
	QFrame *vline = new QFrame;
	vline->setFrameShape(QFrame::VLine);
	vline->setFrameShadow(QFrame::Raised);
	QFrame *hline = new QFrame;
	hline->setFrameShape(QFrame::HLine);
	hline->setFrameShadow(QFrame::Raised);
	// Lines end
	
	// Layouts start
	QHBoxLayout *fanLayout[fanList.size()];
	for (int i = 0; i < fanList.size(); i++) {
		fanLayout[i] = new QHBoxLayout;
		fanLayout[i]->addWidget(fanLbl[i]);
		fanLayout[i]->addWidget(fanSpin[i]);
	}

	QVBoxLayout *fan = new QVBoxLayout;
	for (int i = 0; i < fanList.size(); i++)
		fan->addLayout(fanLayout[i]);
	fan->addItem(vspacer);
	
	QVBoxLayout *temp = new QVBoxLayout;
	for (int i = 0; i < tempList.size(); i++)
		temp->addWidget(tempLbl[i]);
	temp->addItem(vspacer2);
	
	QHBoxLayout *hbox = new QHBoxLayout;
	hbox->addLayout(fan);
	hbox->addWidget(vline);
	hbox->addLayout(temp);
	
	QVBoxLayout *vbox = new QVBoxLayout;
	vbox->addLayout(hbox);
	vbox->addWidget(hline);
	
	widget->setLayout(vbox);
	// Layouts end

	// Timers start
	QTimer *read = new QTimer;
	connect(read, SIGNAL(timeout()), this, SLOT(readValues()));
	read->start(500);
	// Timers end

	if (startMinimized == 1)
		showMinimized();
}

void MainWindow::fanCtrl(int speed)
{
	QSpinBox *spinner = qobject_cast<QSpinBox *>(sender());
	for (int i = 0; i < fanList.size(); i++) {
		if (spinner == fanSpin[i]) {
			QString pwmPath = tr("%1/pwm%2").arg(path).arg(i+1);
			QFile *pwmF = new QFile(pwmPath);
			if( pwmF->open(QIODevice::WriteOnly ) ) {
				QTextStream stream( pwmF );
				stream << speed;
				pwmF->close( );
			}
			break;
		}
	}
}

void MainWindow::preferencesDialog()
{
	PreferencesDialog *preferences;
	preferences = new PreferencesDialog(fanList, tempList);
	preferences->exec();
	readSettings();
}

void MainWindow::createTrayIcon()
{
	#if QT_VERSION >= 0x40200
		trayMenu = new QMenu;
		trayMenu->addAction("Visa", this, SLOT(show()));
		trayMenu->addAction(QString::fromUtf8("Göm"), this, SLOT(hide()));
		trayMenu->addSeparator();
		trayMenu->addAction("Quit", qApp, SLOT(quit()));
		trayIcon = new QSystemTrayIcon;
		//trayIcon->setToolTip("QtFan");
		trayIcon->setContextMenu(trayMenu);

		drawTrayIcon(0);
		trayIcon->show();
		connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), 
				this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
	#endif
}

void MainWindow::drawTrayIcon(int temperature)
{
#if QT_VERSION >= 0x40200
			QPixmap pix(22,22);
			QPainter p;
			QFont f("DejaVu Sans Mono", 15);
			QString str = tr("%1").arg(temperature);
			
			p.begin( &pix );
			p.setFont(f);
			p.drawPixmap(0,0,pix);
			pix.fill(Qt::white);
			p.setPen(Qt::blue);
			p.drawText(0,0,22,22,Qt::AlignCenter,str);
			p.end();

			//pix.setAlphaChannel(pix.alphaChannel()); // Not working anymore
			//pix.setMask(pix.createMaskFromColor(Qt::white)); // masked out area not clickable
			trayIcon->setIcon(QIcon(pix));
#endif
}

void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
{
#if QT_VERSION >= 0x40200
	switch (reason) {
		case QSystemTrayIcon::Unknown:
			break;
		case QSystemTrayIcon::Context:
			break;
		case QSystemTrayIcon::MiddleClick:
			break;
		case QSystemTrayIcon::DoubleClick:
			break;
		case QSystemTrayIcon::Trigger:
			if (isHidden())
				showNormal();
			else
				hide();
			break;
	}
#endif
}

void MainWindow::findPath()
{
	QDir d("/sys/class/hwmon/");
	QDir subd;
	if ( d.exists() ) {
		d.setFilter(QDir::Dirs|QDir::NoDotAndDotDot);
		d.setSorting(QDir::Name);
		for (unsigned int i = 0; i < d.count(); i++) {
			subd = d.canonicalPath() + "/" + d[i] + "/device/";
			subd.setFilter(QDir::Files|QDir::NoDotAndDotDot);
			subd.setSorting(QDir::Name);
			for (unsigned int j = 0; j < subd.count(); j++) {
				if ((QDir::match(subd.canonicalPath() + "/temp*_input", subd.canonicalPath() + "/" + subd[j]))) {
					QFile i2cFile(subd.canonicalPath() + "/" + subd[j]);
					if (i2cFile.open(QIODevice::ReadOnly)) {
						QTextStream stream(&i2cFile);
						QString s = stream.readLine();
						i2cFile.close();
						if (!s.startsWith("-")) 
							tempList.append(subd.canonicalPath() + "/" + subd[j]);
					}
				}
			}
			for (unsigned int k = 0; k < subd.count(); k++) {
				if ((QDir::match(subd.canonicalPath() + "/fan*_input", subd.canonicalPath() + "/" + subd[k]))) {
					QFile i2cFile(subd.canonicalPath() + "/" + subd[k]);
					if (i2cFile.open(QIODevice::ReadOnly)) {
						QTextStream stream(&i2cFile);
						QString s = stream.readLine();
						i2cFile.close();
						if (!s.startsWith("-")) 
							fanList.append(subd.canonicalPath() + "/" + subd[k]);
					}
				}
			}
		path = subd.canonicalPath();
		}
	}
}

void MainWindow::readValues()
{
	if (fanList.size() >= 1 ) {
		int rpm[fanList.size()];
		for (int i = 0; i < fanList.size(); i++)
		{
			// Fan Speeds
			QFile fanspeed(fanList.at(i));
			if( !fanspeed.open(QIODevice::ReadOnly))
				break;
			QTextStream stream( &fanspeed );
			stream >> rpm[i];
			fanLbl[i]->setText(tr("%1: %2").arg(fan_name[i]).arg(rpm[i]));
			fanspeed.close();
		}
	}

	if (tempList.size() >= 1) {
		double tempDbl[tempList.size()];
		for (int i = 0; i < tempList.size(); i++)
		{
			// Temperatures
			QFile tempF(tempList.at(i));
			if( !tempF.open(QIODevice::ReadOnly))
				break;
			QTextStream stream2( &tempF );
			stream2 >> tempDbl[i];
			tempDbl[i] /= 1000;
			tempLbl[i]->setText(QString::fromUtf8("%1: %2°C").arg(temp_name[i]).arg(tempDbl[i]));
			tempF.close();
		}
	drawTrayIcon((int)tempDbl[trayTemp]);
	}
}

void MainWindow::readSpinners()
{
	int pwm[fanList.size()];
	for (int i = 0; i < fanList.size(); i++)
	{
		QString pwmPath = tr("%1/pwm%2").arg(path).arg(i+1);
		QFile pwmF(pwmPath);
		if (!pwmF.open(QIODevice::ReadOnly))
			break;
		QTextStream stream(&pwmF);
		stream >> pwm[i];
		fanSpin[i]->setValue(pwm[i]);
		pwmF.close();
	}
}

void MainWindow::closeEvent(QCloseEvent *event)
{
#if QT_VERSION >= 0x40200
	if (closeToTray == 0) {
		writeSettings();
		event->accept();
	} else {
		event->ignore();
		hide();
	}
#else
	writeSettings();
	event->accept();
#endif
}

void MainWindow::hideEvent(QHideEvent *event)
{
	event->accept();
#if QT_VERSION >= 0x40200
	if (minimizeToTray == 1)
		hide();
#endif
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("About"), tr("<b><i><u>QtFan</i></u></b>"));
}

void MainWindow::createActions()
{
	exitAct = new QAction(tr("E&xit"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setStatusTip(tr("Exit the application"));
	connect(exitAct, SIGNAL(triggered()), qApp, SLOT(quit()));

	aboutAct = new QAction(tr("&About"), this);
	aboutAct->setStatusTip(tr("Show the application's About box"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	aboutQtAct = new QAction(tr("About &Qt"), this);
	aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
	connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	prefAct = new QAction(tr("&Preferences"), this);
	prefAct->setStatusTip(tr("Show the preferences dialog"));
	connect(prefAct, SIGNAL(triggered()), this, SLOT(preferencesDialog()));
}

void MainWindow::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(prefAct);
	fileMenu->addAction(exitAct);

	menuBar()->addSeparator();

	helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(aboutAct);
	helpMenu->addAction(aboutQtAct);
}

void MainWindow::createStatusBar()
{
	statusBar()->showMessage(tr("Ready"));
}

void MainWindow::readSettings()
{
	QSettings *settings = new QSettings("QtFan");
	if (firstStart) {
//		QPoint pos = settings->value("pos").toPoint();
		move(settings->value("pos").toPoint());
//		move(pos);
		firstStart = false;
	}
	for (int i = 0; i < 8; i++) {
		fan_name[i] = settings->value(tr("fan_name%1").arg(i)).toString();
		temp_name[i] = settings->value(tr("temp_name%1").arg(i)).toString();
		if (fan_name[i] == "") 
			fan_name[i] = tr("Fan %1").arg(i+1);
		if (temp_name[i] == "")
			temp_name[i] = tr("Temp %1").arg(i+1);
	}
	closeToTray = settings->value("closeToTray").toInt();
	minimizeToTray = settings->value("minimizeToTray").toInt();
	startMinimized = settings->value("startMinimized").toInt();
	trayTemp = settings->value("trayTemp").toInt();
}

void MainWindow::writeSettings()
{
	QSettings *settings = new QSettings("QtFan");
	settings->setValue("pos", pos());
}

