/*   This file is part of QtFan.

    QtFan is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2

    QtFan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with QtFan.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

class QAction;
class QMenu;
class QLabel;
class QSpinBox;
#if QT_VERSION >= 0x40200
#include <QSystemTrayIcon>
#endif

class MainWindow : public QMainWindow
{
	Q_OBJECT

	public:
		MainWindow();

	protected:
		void closeEvent(QCloseEvent *);
		void hideEvent(QHideEvent *);

	private slots:
		void about();
		void readValues();
		void readSpinners();
		void findPath();
		void trayActivated(QSystemTrayIcon::ActivationReason);
		void preferencesDialog();
		void fanCtrl(int);
	
	private:
		void createActions();
		void createMenus();
		void createStatusBar();
		void readSettings();
		void writeSettings();
		void createTrayIcon();
		void drawTrayIcon(int);

		QMenu *fileMenu;
		QMenu *helpMenu;
		
		QLabel *fanLbl[8];
		QLabel *tempLbl[8];

		QSpinBox *fanSpin[8];
		
		QAction *exitAct;
		QAction *aboutAct;
		QAction *aboutQtAct;
		QAction *prefAct;

		int closeToTray;
		int minimizeToTray;
		int startMinimized;
#if QT_VERSION >= 0x40200
		QSystemTrayIcon *trayIcon;
		QMenu *trayMenu;
#endif
	
		QList<QString> tempList;
		QList<QString> fanList;
};

#endif
