/*   This file is part of QtFan.

    QtFan is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2

    QtFan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with QtFan.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>

#include "preferencesdialog.h"

QList<QString> fanList, tempList;
int comboIndex;

PreferencesDialog::PreferencesDialog(QList<QString> fanList1, QList<QString> tempList1)
{
	fanList = fanList1;
	tempList = tempList1;

	setWindowTitle(tr("Preferences"));

	for (int i = 0; i < fanList.size(); i++)
		fanName[i] = new QLineEdit();
	for (int i = 0; i < tempList.size(); i++)
		tempName[i] = new QLineEdit();

	readSettings();

	minimizeTray = new QCheckBox(tr("Minimize to tray"));
	if (minimizeToTray == 1)
		minimizeTray->setCheckState(Qt::Checked);
	
	closeTray = new QCheckBox(tr("Close to tray"));
	if (closeToTray == 1)
		closeTray->setCheckState(Qt::Checked);

	startMinimizedChk = new QCheckBox(tr("Start minimized"));
	if (startMinimized == 1)
		startMinimizedChk->setCheckState(Qt::Checked);

	trayTemp = new QLabel(tr("Temp to show in tray "));
	trayCombo = new QComboBox;

	QSpacerItem *vspacer = new QSpacerItem(20,40,QSizePolicy::Minimum, QSizePolicy::Expanding);

	QVBoxLayout *fan = new QVBoxLayout;
	for (int i = 0; i < fanList.size(); i++)
		fan->addWidget(fanName[i]);
	fan->addItem(vspacer);
	
	QVBoxLayout *temp = new QVBoxLayout;
	for (int i = 0; i < tempList.size(); i++) {
		temp->addWidget(tempName[i]);
		trayCombo->addItem(tempName[i]->text());
	}
	temp->addItem(vspacer);

	trayCombo->setCurrentIndex(comboIndex);

	QHBoxLayout *trayLayout = new QHBoxLayout;
	trayLayout->addWidget(trayTemp);
	trayLayout->addWidget(trayCombo);
	
	QHBoxLayout *fan_temp = new QHBoxLayout;
	fan_temp->addLayout(fan);
	fan_temp->addLayout(temp);
	
	ok_cancel = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
	connect(ok_cancel, SIGNAL(accepted()), this, SLOT(accept()));
	connect(ok_cancel, SIGNAL(rejected()), this, SLOT(reject()));
	
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addLayout(fan_temp);
	mainLayout->addWidget(minimizeTray);
	mainLayout->addWidget(closeTray);
	mainLayout->addWidget(startMinimizedChk);
	mainLayout->addLayout(trayLayout);
	mainLayout->addWidget(ok_cancel);
	setLayout(mainLayout);
}

void PreferencesDialog::accept()
{
	writeSettings();
	close();
}

void PreferencesDialog::readSettings()
{
	QSettings *settings = new QSettings("QtFan");
	closeToTray = settings->value("closeToTray").toInt();
	minimizeToTray = settings->value("minimizeToTray").toInt();
	startMinimized = settings->value("startMinimized").toInt();
	for (int i = 0; i < fanList.size(); i++) {
		fanName[i]->setText(tr("%1").arg(settings->value(tr("fan_name%1").arg(i)).toString()));
		if (fanName[i]->text() == "")
			fanName[i]->setText(tr("Fan %1").arg(i+1));
	}
	for (int i = 0; i < tempList.size(); i++) {
		tempName[i]->setText(tr("%1").arg(settings->value(tr("temp_name%1").arg(i)).toString()));
		if (tempName[i]->text() == "")
			tempName[i]->setText(tr("Temp %1").arg(i+1));
	}
	comboIndex = settings->value("trayTemp").toInt();
}

void PreferencesDialog::writeSettings()
{
	QSettings *settings = new QSettings("QtFan");
	if (closeTray->checkState() == Qt::Checked)
		settings->setValue("closeToTray", 1);
	else
		settings->setValue("closeToTray", 0);
	
	if (minimizeTray->checkState() == Qt::Checked)
		settings->setValue("minimizeToTray", 1);
	else
		settings->setValue("minimizeToTray", 0);

	if (startMinimizedChk->checkState() == Qt::Checked)
		settings->setValue("startMinimized", 1);
	else
		settings->setValue("startMinimized", 0);
	
	for (int i = 0; i < fanList.size(); i++)
		settings->setValue(tr("fan_name%1").arg(i), fanName[i]->text());
	for (int i = 0; i < tempList.size(); i++)
		settings->setValue(tr("temp_name%1").arg(i), tempName[i]->text());

	settings->setValue("trayTemp", trayCombo->currentIndex());
}
